#!/usr/local/bin/perl -w

# $Header: /nfs/pathsoft/repository/psu/genexec/perl/src/fasta_to_tab.pl,v 1.1 1999/03/15 15:12:45 kmr Exp $

# purpose: This script will read a fasta sequence file and generate a .tab
# purpose: file (EMBL feature table).  There will be one feature in
# purpose: the feature table for each fasta record.  The note field of each
# purpose: feature will be the header line of the corresponding record.

use strict;

if (@ARGV == 0) {
  die "usage: $0 fasta_file\n";
}

# the record (read so far)
my $record_seq = "";

# the total number of bases in the previous records
my $previous_bases_count = 0;

# the name of the record
my $record_name = undef;

while (<>) {
  if (/^\>(.*)/) {
    if (defined ($record_name)) {
      write_record ($record_name, $record_seq, $previous_bases_count + 1);
      $previous_bases_count += length ($record_seq);
      $record_seq = "";
    }
    $record_name = $1;
  } else {
    chomp;
    $record_seq .= $_;
  }
}

if (defined ($record_name)) {
  write_record ($record_name, $record_seq, $previous_bases_count + 1);
}

sub write_record
{
  my ($record_name, $record_seq, $start_base) = @_;

  my $end_base = $start_base + length ($record_seq) - 1;

  if (!defined ($main::colour) || $main::colour == 11) {
    $main::colour = 10;
  } else {
    $main::colour = 11;
  }

  print <<EOF;
FT   misc_feature    $start_base..$end_base
FT                   /note="$record_name"
FT                   /colour=$main::colour
EOF
}
