#!/nfs/newlocal/bin/perl5.6.0 -w

use strict;

use Bio::PSU::Range;
use Bio::PSU::Seq;
use Bio::PSU::Feature;
use Bio::PSU::IO::EMBL::Stream;
use Bio::PSU::IO::BufferFH;

if ((@ARGV+0) == 0) {
  print STDERR "specify the name of a geneid output file\n";
  exit 0;
}

my $gffFile = $ARGV[0];
my @tmp = split ('\.', $gffFile);
my $cdsFile       = $tmp[0] . ".geneid.cds.tab";
my $otherFeatFile = $tmp[0] . ".geneid.rest.tab";
open GENEID,  "<$gffFile" or die "can't open file, $gffFile\n";
open CDS,     ">$cdsFile" or die "can't open file, $cdsFile\n";

my @features = ();
my $seq      = Bio::PSU::Seq->new (-type => 'dna');
my $bfh      = Bio::PSU::IO::BufferFH->new(-fh => \*CDS);
my $stream   = Bio::PSU::IO::EMBL::Stream->new(-bfh => $bfh);

while (<GENEID>) {
  next if /^##/;
  my $line = $_;
  # print STDERR "new line : $line";

  # extracting the number of exons for the current gene

  if ($line =~ /# Gene \d+\(\w+\)\. (\d+) exons\..+/) {

      my $nb_exons = $1;
      my $i = 1;

      # print STDERR "new gene of $nb_exons exons found\n";

      # CDS ranges
      my @ranges = ();
      # exon scores
      my @scores = ();
      my $cds_score   = 0;
      my $codon_start = undef;

      # build the cds feature by joining the exon together

      while ($i <= $nb_exons) {
	$line = <GENEID>;
	# remove the white spaces just keep the tabulations
	$line =~ s/ //g;
	# print STDERR "feature line : $line\n";
	$line =~ /(\w+)\t+(\S+)\t+(\w+)\t+(\d+)\t+(\d+)\t+(\S+)\t+(\+|-)\t+(\d+)\t+.+/;
	my $seqName     = $1;
	my $nameFeature = $3;
	my $start       = $4;
	my $end         = $5;
	my $score       = $6;
	my $strand      = 1;
	if ($7 =~ /-/) {
	  $strand = -1;
	}
	my $frame       = $8;
	# bug when the strand if "-", can not get the frame in that case !!!
	# so substitute - by + to make the parsing work
	if (not defined $frame) {
	  $line =~ s/-/\+/g;
	  $line =~ /(\w+)\t+(\S+)\t+(\w+)\t+(\d+)\t+(\d+)\t+(\S+)\t+(\+|-)\t+(\d+)\t+.+/;
	  $frame = $8;
	}
	# specify the frame for the first exon, which might not start with a MET
	if (($strand==1 && $i==1) || ($strand==-1 && $i==$nb_exons)) {
	  if ($frame =~ /0/) {
	    $codon_start = 1;
	  }
	  elsif ($frame =~ /1/) {
	    $codon_start = 2;
	  }
	  elsif ($frame =~ /2/) {
	    $codon_start = 3;
	  }
	}

	$seq->id ($seqName);
  
	my $range = Bio::PSU::Range->new(-start       => $start,
					 -end         => $end,
					 -strand      => $strand);
	push (@ranges, $range);
	push (@scores, $score);
	$cds_score = $cds_score + $score;

	$i++;
      }
      
      # build the feature object
      
      my $feature = Bio::PSU::Feature->new(-key    => 'CDS',
					   -ranges => \@ranges);
      my $note = "GeneID - exon scores: " . join (', ', @scores);
      $feature->qadd('note', $note);
      $feature->qadd('method', 'GENEID');
      $feature->qadd('colour', '3');
      $feature->qadd('score', $cds_score);
      $feature->qadd('note', 'exon scores, associated with the location ranges - always from left to right');
      $feature->qadd('codon_start', $codon_start);
      
      push (@features, $feature);
  }
  else {
    # print STDERR "next execution shouldn't occur\n";
    # next;
  }
}

$seq->features (@features);
$stream->write_seq($seq);
