#!/usr/local/bin/perl5.6.0 -w

# $Header: /nfs/pathsoft/repository/psu/genexec/perl/src/map_protein_to_contigs.pl,v 1.2 2001/11/09 10:29:12 kmr Exp $

# purpose: This script takes an MSPcrunch file (which the result of the
# purpose: tblastn of a protein database and a FASTA database of contigs)
# purpose: and uses it to pick contigs that match from the contig database.

use strict;

use Bio::PSU::Seq;
use Bio::PSU::SeqFactory;
use Bio::PSU::IO::BufferFH;

sub usage
{
  die <<EOF;
usage:
  $0 contig_database mspcrunch_file
EOF
}

if (!@ARGV) {
  usage;
}

my $debug = 0;

if ($ARGV[0] eq "-d") {
  $debug = 1;
  shift;
}

if (@ARGV != 2) {
  usage;
}

if (!$debug) {
  open STDERR, ">/dev/null";
}

my %seq_hash = ();


if (!$debug) {
  my $seqfactory =
    Bio::PSU::SeqFactory->make(-file   => $ARGV[0],
                               -format => "FASTA",
                               -type => 'dna');

  while (my $seq = $seqfactory->next_seq) {
    my $id = $seq->id ();

    $seq_hash{$id} = $seq;
  }
}



open MSPCRUNCH, "$ARGV[1]" or die "can't open $ARGV[1]\n";

my @q_names_in_order = ();

my %hits = ();

my %rev_hits = ();

my @hits = ();

while (<MSPCRUNCH>) {
  my @bits = split /\s+/, $_;

  my ($score,$percent_id,$q_start,$q_end,$q_name,$s_start,$s_end,$s_name) = @bits;

  $q_name =~ s/,$//;

  my $rev_comp = 0;

  if ($q_start > $q_end) {
    $rev_comp = 1;

    ($q_start, $q_end) = ($q_end, $q_start);
  }

  my $new_hit = {
                 q_name => $q_name,
                 s_name => $s_name,
                 q_start => $q_start,
                 q_end => $q_end,
                 rev_comp => $rev_comp,
                 score => $score
                };

  if (!exists $hits{$q_name}) {
    push @q_names_in_order, $q_name;
  }

  push @hits, $new_hit;

  push @{$hits{$q_name}}, $new_hit;

#    if ($s_name =~ /8972/) {
#      print STDERR "$s_name $rev_comp \n";
#    }

  push @{$rev_hits{$s_name}}, $new_hit;

  print STDERR "pushed $score, $q_start, $q_end, $q_name, $rev_comp, $s_name, $new_hit, $rev_comp\n";
}

#  for my $s_name (keys %rev_hits) {
#    while (@{$rev_hits{$s_name}} > 1) {
#      # remove all but the highest score

#      my $hit0_q_name = $rev_hits{$s_name}[0]{q_name};
#      my $hit1_q_name = $rev_hits{$s_name}[1]{q_name};

#      if (@{$hits{$hit0_q_name}}) {
#      }

#      if ($rev_hits{$s_name}[0]{score} > $rev_hits{$s_name}[1]{score}) {
#        splice @{$rev_hits{$s_name}}, 1, 1;
#      } else {
#        splice @{$rev_hits{$s_name}}, 0, 1;
#      }
#    }
#  }


my @extra_hits = ();


sub remove_hit
{
  my $hit = $_[0];

  @hits = grep { $_ ne $hit } @hits;

  for my $s_name (keys %rev_hits) {
    my @this_s_hits = @{$rev_hits{$s_name}};

    @{$rev_hits{$s_name}} = grep {$_ ne $hit} @this_s_hits;
  }

  for my $q_name (keys %hits) {
    my @this_q_hits = @{$hits{$q_name}};

    @{$hits{$q_name}} = grep {$_ ne $hit} @this_q_hits;
  }
}


# if any contig has more than one hit, remove all but the top scoring hit
for my $s_name (keys %rev_hits) {
  print STDERR "exmining hits to $s_name\n";

  my @this_s_hits = @{$rev_hits{$s_name}};

  my $max_score = -1;
  my $max_hit = undef;

  for (my $i = 0 ; $i < @this_s_hits ; ++$i) {
    my $hit = $this_s_hits[$i];
    my $q_name = $hit->{q_name};
    my $this_score = $hit->{score};

    print STDERR "hit $hit\n";

    if ($this_score > $max_score) {
      $max_score = $this_score;
      $max_hit = $hit;
      print STDERR "max: $this_score $i $hit from ",$hit->{q_name}," to ",
      $s_name, "\n";
    } else {
      print STDERR "not max: $this_score $i $hit ",$hit->{q_name}," to ",
      $s_name, "\n";
    }

    print STDERR "$i $hit\n";
  }

  for (my $i = @this_s_hits - 1 ; $i >= 0 ; --$i) {
    my $hit = $this_s_hits[$i];

    print STDERR "\$i: $i $hit\n";

    print STDERR "$max_hit\n";
    print STDERR "$hit != $max_hit\n";

    if ($hit != $max_hit) {
      my $hit_q_name = $hit->{q_name};

      print STDERR "removing $hit_q_name $hit\n";

      #      print STDERR "new \@this_q_hits: @this_q_hits\n";

      # remove from full list
      print STDERR "before: ", scalar (@hits), "\n";
      remove_hit ($hit);
      print STDERR "after: ", scalar (@hits), "\n";
    }
  }
}



# if a protein hits more than one contig remove all but the hit with the
# maximum score (unless one of the other hits is the only hit to it's contig
# in which case save it in @extra_hits)
if (0) {
  for my $q_name (keys %hits) {
    print STDERR "exmining hits from $q_name\n";

    my @this_q_hits = @{$hits{$q_name}};

    my $max_score = -1;
    my $max_hit = undef;

    print STDERR "\@this_q_hits: @this_q_hits\n";

    for (my $i = 0 ; $i < @this_q_hits ; ++$i) {
      my $hit = $this_q_hits[$i];
      my $s_name = $hit->{s_name};
      my $this_score = $hit->{score};

      if ($this_score > $max_score) {
        $max_score = $this_score;
        $max_hit = $hit;
        print STDERR "max: $this_score $i $hit ",$hit->{s_name},"\n";
      } else {
        print STDERR "not max: $this_score $i $hit ",$hit->{s_name},"\n";
      }

      print STDERR "$i $hit\n";
    }

    for (my $i = @this_q_hits - 1 ; $i >= 0 ; --$i) {
      my $hit = $this_q_hits[$i];

      print STDERR "\$i: $i $hit\n";

      print STDERR "$max_hit\n";
      print STDERR "$hit != $max_hit\n";

      if ($hit != $max_hit) {
        my $hit_s_name = $hit->{s_name};

        print STDERR "removing $hit_s_name $hit\n";

        if (@{$rev_hits{$hit_s_name}} <= 1) {
          # save the hit since it is the last hit to the contig
          push @extra_hits, $hit;
        }

        #      print STDERR "new \@this_q_hits: @this_q_hits\n";

        # remove from full list
        print STDERR "before: ", scalar (@hits), "\n";
        remove_hit ($hit);
        print STDERR "after: ", scalar (@hits), "\n";
      }
    }

    @{$hits{$q_name}} = @this_q_hits;
  }
}

#  my @out_hits = ();
#  my %out_hits = ();

#  for my $hit (@hits) {
#    my $q_name = $hit->{q_name};
#    my $s_name = $hit->{s_name};
#    my $score = $hit->{score};

#    if (defined $out_hits{$s_name}) {
#      push @extra_hits, $hit;
#    } else {
#      push @out_hits, $hit;
#      $out_hits{$s_name}++;
#    }

#  #  print STDERR "$q_name $s_name $score\n";
#  }

#  push @out_hits, @extra_hits;

my $out_seq = Bio::PSU::SeqFactory->make(-fh     => \*STDOUT,
                                         -format => 'fasta');

for my $q_name (@q_names_in_order) {
  my @this_q_hits = @{$hits{$q_name}};

  my $rev_query = 0;

  if ($q_name =~ /\.comp$/) {
    $rev_query = 1;
  }

  # sort by first query base
  @this_q_hits = sort {
    if ($rev_query) {
      $b->{q_end} <=> $a->{q_end}
    } else {
      $a->{q_start} <=> $b->{q_start}
    }
  } @this_q_hits;

  print STDERR "\@this_q_hits: ";
  print STDERR join " ", map { $_->{q_name} } @this_q_hits;

  print STDERR "\n";

  for my $hit (@this_q_hits) {
    print STDERR "writing: ", $hit->{s_name}, " ($hit)\n";

    if (!defined $seq_hash{$hit->{s_name}}) {
      warn "can't find ",$hit->{s_name},"\n";
    }

    if ($hit->{rev_comp} ^ $rev_query) {
      print STDERR "writing rev_comp ", $hit->{rev_comp}, " ", $rev_query;
    } else {
      print STDERR "writing ", $hit->{rev_comp}, " ", $rev_query;
    }

    print STDERR "\n";

    if (!defined $seq_hash{$hit->{s_name}}) {
      next;
    }

    my $seq = $seq_hash{$hit->{s_name}};

    if ($hit->{rev_comp} ^ $rev_query) {
      $out_seq->write_seq ($seq->revcom);
    } else {
      $out_seq->write_seq ($seq);
    }
  }
}
