#!/usr/local/bin/perl --

# $Header: /nfs/pathsoft/repository/psu/genexec/perl/src/renumber_features.pl,v 1.2 1999/07/28 12:39:40 kmr Exp $

# purpose: This script reads an embl entry and will change the systematic
# purpose: gene names in all the /label, /gene and /note qualifiers.

use strict;

use Embl;
use EMBL::Parser;

sub usage
{
  die <<EOF;
usage:  $0 -h (for help) or $0 <label_start> <new_label_start> embl_file

This script reads an embl entry and will change the systematic gene names
in all the /label, /gene and /note qualifiers.
(the new entry is written to standard output)

label_start - should be a perl regular expression that will match the start
 of all current gene names.  eg. if your file contains names looking like
 SC12A3.01 and SC45B5.02 a suitable value for label_start is 'SC[A-Z\d]+'
 You need only match the part to the left of the decimal point.  The single
 quotes will be need if the pattern contains any special characters (like \ )

new_label_start - should a string specifying the new start of the names 
 (this will replace the characters matched by label_start)

embl_file - the name of the embl entry to read


Example:
   $0 SC SCcontig1 my_embl_file
will write in new embl entry to standard output after changing renaming
all the gene names.  The name SC12A3.45 in any qualifier will be changed
to look like SCcontig1.0001 (with a different number for each CDS).
EOF
}

if ($ARGV[0] eq "-h") {
  usage ();
}

if (@ARGV != 3) {
  usage ();
}

open IN_FILE, "$ARGV[2]" or die "$0: can't open $ARGV[2]\n";

open ERRORS, ">>renumber_features_errors";

# this (probably 2 letter) organism code
my $current_name_start = $ARGV[0];

# this is the start text for the new name
my $new_name_start = $ARGV[1];

# read in some embl files from a stream
my $set = Read Embl(\*IN_FILE);  # read all of the embl entries from that stream.


sub fix_qualifiers
{
  my $f = $_[0];

  # this (probably 2 letter) organism code
  my $current_name_start = $_[1];

  my $new_name_start = $_[2];

  my $cds_number = $_[3];

  for my $qual ($f->qualifiers ()) {
    if ($qual->{name} =~ /^(gene|note|label)$/) {
      my $number_string = "$cds_number";

      # pad with zeros so that $number_string is always 4 characters long
      $number_string = ("0" x (4 - length $number_string)) . $number_string;

      my $value = $qual->value;

      $value =~ 
        s/^(\"?)$current_name_start\.\d+(c?)\b/$1$new_name_start.$number_string$2/;

      $qual->value ($value);
    }
  }    
}

# get an array of all of the EMBL::Entry objects within $set
my @entries = $set->entries();
while (my $entry = pop @entries) {
  my $ft = $entry->FT ();

  my @features = $ft->features ();

  my $cds_number = 1;

  # loop over the features in the current entry
  FEATURE: for my $f (@features) {
    if ($f->key () ne "CDS") {
      # go straight to the next feature - we are only interested in
      # CDS features
      next;
    }

    fix_qualifiers ($f, $current_name_start, $new_name_start, $cds_number);

    ++$cds_number;
  }

  $entry->toStream (\*STDOUT);
}

