#!/usr/local/bin/perl -w

# $Header: /nfs/pathsoft/repository/psu/genexec/perl/src/restrict_to_tab.pl,v 1.1 2000/03/01 15:33:24 kmr Exp $

# purpose: Creates a .tab file with common restriction enzymes marked
# purpose: These are the enzymes:
# purpose: HindIII BamHI EcoRI NotI XbaI SalI SacI SacII AseI SfiI
# purpose: PstI ClaI NruI MluI KpnI SmaI Sau3AI MspI HpaII XhoI PvuI PvuII AluI
# purpose: BglII DpnI MboII MboI NarI DraI PacI PmeI EagI EcoRV HincII AccI
# purpose: SphI SwaI

use strict;

sub usage 
{
  die "usage: $0 [-enzymes <enzyme list>] sequence\n";
}

my $in_file;

my $output_key = "misc_feature";

my $default_enzymes = "HindIII,BamHI,EcoRI,NotI,XbaI,SalI,SacI,SacII,AseI,SfiI,PstI,ClaI,NruI,MluI,KpnI,SmaI,Sau3AI,MspI,HpaII,XhoI,PvuI,PvuII,AluI,BglII,DpnI,MboII,MboI,NarI,DraI,PacI,PmeI,EagI,EcoRV,HincII,AccI,SphI,SwaI";
my $enzymes = $default_enzymes;

if (@ARGV && $ARGV[0] eq "-enzymes") {
  shift;

  if (@ARGV) {
    $enzymes = shift;
  } else {
    warn "$0: -enzymes needs an argument\n";
    usage;
  }
}

if (@ARGV) {
  $in_file = shift @ARGV;
} else {
  $in_file = "stdin";
}

if (@ARGV) {
  warn "$0: too many arguments\n";
  usage;
}

open RESTRICT, "restrict $in_file -sitelen 2 -enzymes \"$enzymes\" -stdout -auto|" or
  die "can't start restrict\n";

while (<RESTRICT>) {
  if (/(\d+)\s+(\w+)\s+(\w+)\s+(\d+)\s+(\d+)/) {
    my $name          = $2;
    my $bases_matched = $3;
    my $start_of_site = $1;
    my $end_of_site   = $1 + length ($bases_matched) - 1;

    my $location = "$start_of_site..$end_of_site";
 
    printf "FT   %-15.15s $location\n", $output_key;
    print <<EOF;
FT                   /note="$name site matched $bases_matched"
EOF
  }
}
