#!/usr/local/bin/perl -w

# Run clustalw on a directory with fasta files - if clustalw has already run on a file, it doesn't run again on it.

=head1 NAME

run_clustalw.pl - a clustalw alignment process wrapper script

=head1 SYNOPSIS

  run_clustalw

=head1 DESCRIPTION

  This script wraps an alignment process on a set of fasta files. These files are in the current directory. Each fasta file is a set of sequences on which the alignment is done with clustalw.
  If the aln file has been already generated, the script doesn't run clustalw again on the corresponding fasta file.

  @author axk

=cut

=head1 ALGORITHM

=cut

####################
# import packages
####################

use strict;

my $cur_dir = qx/pwd/;
chomp ($cur_dir);
$cur_dir =~ s/\/tmp_mnt//g;
print STDERR "current dir : $cur_dir\n";

my $machine = "pcs2b";

run_clustalw ($cur_dir, $machine);


##############
# END
##############

sub run_clustalw {
  
  my ($cur_dir, $machine) = @_;
    
  print "running clustalw on each contig to generate the alignment file...\n";
  
  # output directory files
  opendir (THISDIR, $cur_dir) or die "can not read this directory $cur_dir ...";
  my @files_tmp = grep {$_ =~ /^cluster.+fasta/} readdir THISDIR;
  closedir THISDIR;

  print STDOUT "files nb: \n" . (@files_tmp+0) . "\n";

  # filter based on the already existing aln files
  # if already an aln file, don't run clustalw on this cluster

  my @files = ();
  foreach my $file_name (@files_tmp) {
    my @tmp = split (/\./, $file_name);
    my $cluster_name = $tmp[0];
    my $aln_file_name = $cluster_name . ".aln";
    # print "aln file : $aln_file_name\n";
    my $aln_file_name_path = $cur_dir . "/" . $aln_file_name;
    if (not -f $aln_file_name_path) {
      push (@files, $file_name);
    }
  }

  print STDOUT "files nb: \n" . (@files+0) . "\n";

  my @files_t = ();
  foreach my $file_name (@files) {
    my @tmp = split (/\./, $file_name);
    my $cluster_name = $tmp[0];
    # print "cl name: $cluster_name\n";
    my $cluster_number = substr ($cluster_name, 7, length ($cluster_name)) + 0;
    # print STDOUT "cluster_number: $cluster_number\n";
    my $aln_file_name = $cluster_name . ".aln";
    my $contig_name = "CONTIG" . $cluster_number;
    my $contig_file_name = $contig_name . ".fasta";

    print STDERR "command: lsrun -v clustalw -infile=" . $cur_dir . "/" . $file_name . " -outfile= " . $cur_dir . "/" . "$aln_file_name -outorder=INPUT -type=DNA\n";
    # my @args = ("lsrun -v clustalw -infile=" . $cur_dir . "/" . $file_name . " -outfile= " . $cur_dir . "/" . "$aln_file_name -outorder=INPUT -type=DNA");
    my @args = ("lsrun -v clustalw -infile=" . $cur_dir . "/" . $file_name . " -outfile= " . $cur_dir . "/" . "$aln_file_name -outorder=INPUT -type=DNA");
    system (@args);
  }

  print "alignment files done\n these files (*.aln) can be found in the $cur_dir directory\n";
}
