#!/usr/local/bin/perl

# External Modules

# Constraint variables declaration before using them
use strict;

use Cwd;

# Bio modules
# PSU
use Bio::PSU::SeqFactory;
##########################################

=head1 NAME

 - sequence_filtering.pl input_fasta_file [size]

=head1 SYNOPSIS


=head1 DESCRIPTION

  this script filters a set of sequences according to their length - from a fasta imput file, creates another fasta file. The filter per default is 500 pb but you can specify your own length. 

  The size is optional - 500 pb per default

  @author axk

=cut

my $size = 500;
my $_suffix = ".filtered";
my $input_name = "";
my $output_name = "";
# number of bases per line in the output fasta file
my $size_per_line = 60;

# Check if in the command line there is an input file name
my $lgth = @ARGV + 0;
# if no args, display the help message
if ($lgth == 0 || $lgth > 2) {
  print "specify an input fasta file and no more than two arguments\n";
  exit 1;
}
else {
  $input_name = $ARGV[0];
  if ($lgth == 2) {
    $size = $ARGV[1];
  }
}
my $curdir = cwd;
print "current dir : $curdir\n";
my $input_name_path = $curdir . "/" . $input_name;
$output_name = $input_name . $_suffix;

my $nb_seq = qx/cat $input_name_path | grep \">\" | wc -l/;

print STDERR ("number of sequences to process : $nb_seq\n");

print STDERR ("filtering processing...\n");

my $seqi = Bio::PSU::SeqFactory->make(-file   => "<$input_name_path",
				      -format => 'fasta');
open (FILEOUT, "> $output_name") or die "can't create this file: $!";
my $seqo = Bio::PSU::SeqFactory->make(-fh     => \*FILEOUT, 
				      -format => 'fasta');
my $i = 0;
my $percent_memorized = -1;

while (my $sequence = $seqi->next_seq) {

  # print report - the percentage 0% 5% 10%...

  my $current = int ($i*100/$nb_seq);
  if (($current % 5) == 0 && $current != $percent_memorized) {
    print STDERR "$current% ";
    $percent_memorized = $current;
  }

  my $lgth_seq = length ($sequence->str);
  if ($lgth_seq > $size) {
    $seqo->write_seq($sequence);
  }
  $i++;
}

close FILEOUT;

print STDERR ("\n");
print STDERR ("filtering done.\n");
