#!/usr/local/bin/perl5.6.0 -w

# -*- cperl -*-

# $Header: /nfs/pathsoft/repository/psu/genexec/perl/src/tab_to_dotter.pl,v 1.2 2001/02/21 14:07:18 kdj Exp $

# purpose: This script will turn an EMBL feature table file into a dotter
# purpose: feature file.

use strict;

use Bio::PSU::Feature;
use Bio::PSU::SeqFactory;
use Getopt::Std;
use Pathogens::Artemis;

my %dotter_colours = (
   0 => "WHITE",
   1 => "DARKGRAY",
   2 => "RED",
   3 => "GREEN",
   4 => "BLUE",
   5 => "CYAN",
   6 => "MAGENTA",
   7 => "YELLOW",
   8 => "LIGHTGREEN",
   9 => "LIGHTBLUE",
   10=> "ORANGE",
   11=> "BROWN",
   12=> "LIGHTRED",
   13=> "LIGHTGRAY",
   14=> "BLACK",
   15=> "PALERED",
   16=> "PURPLE",
   17=> "VIOLET",
   18=> "PALEVIOLET",
   19=> "GRAY",
   20=> "PALEGRAY",
   21=> "CERISE",
   22=> "MIDBLUE",
   23=> "PALEORANGE",
   24=> "DARKRED",
   25=> "DARKGREEN",
   27=> "DARKBLUE",
   28=> "PALEGREEN",
   29=> "PALEBLUE",
   30=> "PALEYELLOW",
   31=> "PALECYAN",
   32=> "PALEMAGENTA",
);

my %options = ();

getopts ("v", \%options);

my $file = '-';

if (@ARGV) {
  $file = shift;
}

sub get_dotter_colour ($)
{
  my ($feature) = @_;

  if (defined $feature->colour and exists $dotter_colours{$feature->colour}) {
    return $dotter_colours{$feature->colour};
  } else {
    return "BLACK";
  }
}

sub process_feature ($)
{
  my ($feature) = @_;

  if ($feature->key eq "source") {
    return;
  }

  my $start = $feature->start;
  my $end = $feature->end;

  my $name = $feature->key;

  if (defined $feature->gene) {
    $name = $feature->gene;
  }

  if (defined $feature->label && $feature->label ne "*") {
    $name = $feature->label;
  }

  my $colour = get_dotter_colour ($feature);

  my $direction = '@1';

  if ($options{v}) {
    $direction = '@2';
  }

  print "0 $direction features $start $end $colour $name\n";
}

my $seqfactory =
    Bio::PSU::SeqFactory->make(-file => $file);

my $seq = $seqfactory->next_seq;

if (defined $seq) {
  for my $feature ($seq->features) {
    process_feature ($feature);
  }
} else {
  die "Could not read any features from the tab file\n";
}
