=head1 QualifierEditor::GO

QualifierEditor::App - This package contains the GO reader for the
qualifier_editor application (aka. MESS).

$Header: /nfs/pathsoft/repository/psu/genlib/perl/src/QualifierEditor/GO.pm,v 1.2 2001/11/05 14:10:11 kmr Exp $

=cut

use strict;
use Exporter;
use Carp;

use vars qw(@ISA @EXPORT_OK);

@ISA = qw(Exporter);

#@EXPORT_OK = qw (get_go_descriptions get_go_mapping);

my $go_flattened_file = "/nfs/disk222/yeastpub/analysis/pathogen/GO/go.flat";
my $go_mapping =
  "/nfs/disk222/yeastpub/analysis/pathogen/GO/gene_association.sgd";

# indexed by GO id, contents are a description and a name:
#   $go_flattened{GO:012345}{description}
#   $go_flattened{GO:012345}{name}
my %go_flattened = ();

# indexed by gene id
my %mapping = ();

sub read_flattened_file
{
  open FILE, $go_flattened_file or die "can't open $go_flattened_file\n";

  while (my $line = <FILE>) {
    if ($line =~ /^(GO:\d+)\t([^\t]*)\t(.*)/) {
      if (1 || exists $go_flattened{$1}) {
        # only save the ids that are annotated in the mappings file
        if (length $2) {
          $go_flattened{$1}{name} = $2;
        } else {
          $go_flattened{$1}{name} = undef;
        }
        if (length $3) {
          $go_flattened{$1}{description} = $3;
        } else {
          $go_flattened{$1}{description} = undef;
        }
      }
    }
  }

  if (keys %go_flattened == 0) {
    die "failed to read anything from $go_flattened_file\n";
  }

  close FILE;
}

# Read the mapping file into %mapping.  The key of the %mapping hash is
# the database id of the protein.  The value of the hash is an array of hashes
# of this form {go_id=>"GO:0005743", evidence_code=>"ISS"}.  Ontology codes
# (F, P, C) are added to the %go_flattened hash: {GO:0005743}{ontology=>"F"}
sub read_mappings
{
  open FILE, $go_mapping or die "can't open $go_mapping\n";

  while (my $line = <FILE>) {
    if ($line =~ /^([^\t]+)\t  # database name
                   ([^\t]+)\t # ID
                   ([^\t]*)\t   # gene name
                   ([^\t]*)\t   # dunno
                   ([^\t]+)\t # GO ID
                   ([^\t]*)\t # internal database information
                   ([^\t]*)\t # evidence code
                   ([^\t]*)\t # might be NO meaning the not this GO id
                   ([^\t])    # ontology tag (F,C or P)
                  /x) {
      my $database_name = $1;
      my $db_id = $2;
      my $gene_name = $3;
      my $go_id = $5;
      my $internal_db_info = $6;
      my $evidence_code = $7;
      my $no_flag = $8;
      my $ontology_tag = $9;

      if ($no_flag !~ /no/i) {
        push @{$mapping{$db_id}}, {go_id => $go_id,
                                   evidence_code => $evidence_code,
                                   database_name => $database_name,
                                   };

        $go_flattened{$go_id}{ontology} = $ontology_tag;
      }
    }
  }

  if (keys %mapping == 0) {
    die "failed to read anything from $go_mapping";
  }

  close FILE;
}


sub read_all
{
  read_mappings;
  read_flattened_file;
}

# return the description of the given GO id.  returns undef if the id
# doesn't (yet) have a description
sub get_go_description
{
  my $go_id = shift;

  if (keys %go_flattened == 0) {
    read_all;
  }

  return $go_flattened{$go_id}{description};
}

# return the name of the given GO id.  returns undef if the id
# doesn't have a name.
sub get_go_name
{
  my $go_id = shift;

  if (keys %go_flattened == 0) {
    read_all;
  }

  return $go_flattened{$go_id}{name};
}

# return the ontology tag of the ontology that contains the given GO id.
# returns C, F or P or undef if we don't know.
sub get_go_ontology_name
{
  my $go_id = shift;

  if (keys %go_flattened == 0) {
    read_all;
  }

  return $go_flattened{$go_id}{ontology};
}

# Return an array containing all the go ids associated with the given
# gene_id. # Each element of the array is a hash contains a go id and
# a description: {go_id => "GO:0005743", evidence_code => "ISS"}
sub get_go_mapping
{
  my $gene_id = shift;

  if (keys %mapping == 0) {
    read_all;
  }

  return @{$mapping{$gene_id}};
}

1;
